@@ -234,7 +234,7 @@ delete_archive_members (void)
   if (logical_status == HEADER_SUCCESS
       || logical_status == HEADER_SUCCESS_EXTENDED)
     {
-      write_archive_to_stdout = 0;
+      write_archive_to_stdout = false;
 
       /* Save away blocks before this one in this record.  */
 
@@ -262,8 +262,7 @@ delete_archive_members (void)
 	    flush_archive ();
 	  status = read_header (false);
 	  
-	  if (extended_header.size)
-	    xheader_decode (&current_stat_info);
+	  xheader_decode (&current_stat_info);
 	  
 	  if (status == HEADER_ZERO_BLOCK && ignore_zeros_option)
 	    {
